/*****************************************************************************
 * Copyright (C) 2004-2009 Christoph Thielecke <crissi99@gmx.de>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This package is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this package; if not, write to the Free Software               *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA *
 *****************************************************************************/

#ifndef TOOLINFO_H
#define TOOLINFO_H

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QProcess>
#include <QtCore/QStringList>

class ToolInfo : public QObject
{
    Q_OBJECT

public:
    /**
     * Information about a helper program
     * @param GlobalConfig Pointer to global config
     * @param Name Name of the tool
     * @return None
     */
    ToolInfo(const QString &Name);

private:
    QProcess *CollectToolInfoProcess;
    QProcess *CollectOpenvpnCapabilitiesProcess;
    QStringList SearchPathList;
    bool collectRunning;
    bool collectOpenvpnCapabilitiesRunning;
    QStringList env;

public:
    QString Name; //< Name of the tool
    QString Version; //< Version number of the tool
    QString PathToExec; //< Path to executable
    QString Capabilities;
    QString TryPath_first; //< try that path first before list
    bool collectToolInfo(); //< Collects information of the tool
    bool programsInPath;
    bool found;
    void addSearchPath(const QString &path); //< Adds a path for searching
    void removeSearchPath(const QString &path); //< Removes a path for searching
    void getOpenvpnCapabilities();


private slots:
    void readStdOutCollectToolInfo();
    void readStdErrCollectToolInfo();
    void readStdOutCollectOpenvpnCapabilies();
    void readStdErrCollectOpenvpnCapabilies();
    void collectOpenvpnCapabilitiesProcessFinished(int, QProcess::ExitStatus);
};

#endif
